@@ -2,7 +2,7 @@
 """Release information about pymta."""
 
 name = "pymta"
-version = "0.2"
+version = "0.2.1"
 description = "library to build a custom SMTP server"
 long_description = """pymta is a library to build a custom SMTP server in Python. This is useful if 
 you want to...
@@ -14,7 +14,7 @@ you want to...
 author = "Felix Schwarz"
 email = "felix.schwarz@oss.schwarz.eu"
 url = "http://www.schwarz.eu/opensource/projects/pymta"
-download_url = "http://oss.schwarz.eu/cgi-bin/hgwebdir.cg/pymta"
+download_url = "http://www.schwarz.eu/opensource/projects/pymta/download/pymta-%s.tar.gz" % version
 copyright = "© 2008 Felix Schwarz"
 license="MIT"
 
